@@ -28,7 +28,7 @@ from src.utilities import (getTrackingScriptString,
 
 def apbsOpalExec(logTime, form, apbsOptions):
     
-    sys.path.append(os.path.dirname(HAVE_APBS))
+    sys.path.append(os.path.dirname(APBS_LOCATION))
     from ApbsClient import execApbs, initRemoteVars, enoughMemory
 
     #style = "%spdb2pqr.css" # HARDCODED
@@ -72,10 +72,7 @@ def apbsOpalExec(logTime, form, apbsOptions):
 
     #argv=[os.path.abspath("tmp/%s/%s.in") % (logTime, "apbsinput")] # HARDCODED??  
     argv=[os.path.abspath("%s%s%s/apbsinput.in" % (INSTALLDIR, TMPDIR, logTime))]  
-    if DEFAULT_APBS_OPAL_URL == "0":
-        vars={'service_url' : APBS_OPAL_URL}
-    else:
-        vars = None
+    vars={'service_url' : APBS_OPAL_URL}
 
     # Check for enough memory
     if(not enoughMemory(argv[-1])):
@@ -164,7 +161,7 @@ def apbsExec(logTime, form, apbsOptions):
 #        statusfile.close()
 
 
-        apbs_stdin, apbs_stdout, apbs_stderr = os.popen3('%s apbsinput.in' % HAVE_APBS)
+        apbs_stdin, apbs_stdout, apbs_stderr = os.popen3('%s apbsinput.in' % APBS_LOCATION)
 
         startLogFile(logTime, 'apbs_stdout.txt', apbs_stdout.read())
 #        stdoutFile=open('%s%s%s/apbs_stdout.txt' % (INSTALLDIR, TMPDIR, logTime), 'w')
@@ -468,8 +465,8 @@ def generateForm(file, initVars, pdb2pqrID, type):
     print "</table></div></ul>"
 
     print """           <div class=\"mg-para\"><ul>
-                <li>Amount of overlap to include between the individual processors' meshes <a href=\"http://apbs.wustl.edu/MediaWiki/index.php/ELEC_input_file_section#ofrac\" target=\"_blank\"><font title=\"frac\">(<span class=\"tooltip\">?</span>)</font></a>:"""
-    print "<input type=\"text\" name=\"frac\" size=\"10\" maxlength=\"20\""
+                <li>Amount of overlap to include between the individual processors' meshes <a href=\"http://apbs.wustl.edu/MediaWiki/index.php/ELEC_input_file_section#ofrac\" target=\"_blank\"><font title=\"ofrac\">(<span class=\"tooltip\">?</span>)</font></a>:"""
+    print "<input type=\"text\" name=\"ofrac\" size=\"10\" maxlength=\"20\""
     if initVars.has_key('processorMeshOverlap'):
         print "value=\"%f\"" % initVars['processorMeshOverlap']
     print "/></li><br />"
@@ -1215,6 +1212,7 @@ def unpickleVars(pdb2pqrID):
     elif myElec.npbe == 1:
         apbsOptions['solveType'] = 'nonlinearized'
 
+    #TODO: Currently this is not used.
     if len(myElec.ion) == 0:
         apbsOptions['mobileIonSpecies[0]'] = None
     else:
@@ -1283,6 +1281,12 @@ def fieldStorageToDict(form):
         apbsOptions['writeSmol'] = True
     else:
         apbsOptions['writeSmol'] = False
+        
+    if form.has_key("asyncflag") and form["asyncflag"].value == "on":
+        apbsOptions['async'] = locale.atoi(form["async"].value)
+        apbsOptions['asyncflag'] = True
+    else:
+        apbsOptions['asyncflag'] = False
 
     if form.has_key("writesspl") and form["writesspl"].value == "on":
         apbsOptions['writeCheck'] += 1
@@ -1361,7 +1365,9 @@ def fieldStorageToDict(form):
     apbsOptions['pqrFileName'] = form['pdb2pqrid'].value+'.pqr'
 
     #ELEC section variables
-    apbsOptions['calcType'] = form["type"].value  
+    apbsOptions['calcType'] = form["type"].value 
+    
+    apbsOptions['ofrac'] = locale.atof(form["ofrac"].value)
 
     apbsOptions['dimeNX'] = locale.atoi(form["dimenx"].value)
     apbsOptions['dimeNY'] = locale.atoi(form["dimeny"].value)
@@ -1378,6 +1384,10 @@ def fieldStorageToDict(form):
     apbsOptions['glenX'] = locale.atof(form["glenx"].value)
     apbsOptions['glenY'] = locale.atof(form["gleny"].value)
     apbsOptions['glenZ'] = locale.atof(form["glenz"].value)
+    
+    apbsOptions['pdimeNX'] = locale.atof(form["pdimex"].value)
+    apbsOptions['pdimeNY'] = locale.atof(form["pdimey"].value)
+    apbsOptions['pdimeNZ'] = locale.atof(form["pdimez"].value)
 
     if form["cgcent"].value == "mol":
         apbsOptions['coarseGridCenterMethod'] = "molecule"
@@ -1423,12 +1433,15 @@ def fieldStorageToDict(form):
     apbsOptions['calcForce'] = form["calcforce"].value
 
     for i in range(0,3):
-        if form['charge%i' % i].value != "":
-            apbsOptions['mobileIonSpeciesCharge'] = locale.atoi(form['charge%i' % i].value)
-        if form['conc%i' % i].value != "":
-            apbsOptions['mobileIonSpeciesConcentration'] = locale.atof(form['conc%i' % i].value)
-        if form['radius%i' % i].value != "":
-            apbsOptions['mobileIonSpeciesRadius'] = locale.atof(form['radius%i' % i].value)
+        chStr = 'charge%i' % i
+        concStr = 'conc%i' % i
+        radStr = 'radius%i' % i
+        if form[chStr].value != "":
+            apbsOptions[chStr] = locale.atoi(form[chStr].value)
+        if form[concStr].value != "":
+            apbsOptions[concStr] = locale.atof(form[concStr].value)
+        if form[radStr].value != "":
+            apbsOptions[radStr] = locale.atof(form[radStr].value)
     apbsOptions['writeFormat'] = form["writeformat"].value
     #apbsOptions['writeStem'] = apbsOptions['pqrFileName'][:-4]
     apbsOptions['writeStem'] = form["pdb2pqrid"].value
@@ -1471,7 +1484,10 @@ def pqrFileCreator(apbsOptions):
     if apbsOptions['calcType']!="fe-manual":
         input.write('%sdime %d %d %d\n' % (apbsOptions['tab'], apbsOptions['dimeNX'], apbsOptions['dimeNY'], apbsOptions['dimeNZ']))
     if apbsOptions['calcType'] == "mg-para":
-        input.write('%sdime %d %d %d\n' % (apbsOptions['tab'], apbsOptions['pdime'][0], apbsOptions['pdime'][1], apbsOptions['pdime'][2]))
+        input.write('%spdime %d %d %d\n' % (apbsOptions['tab'], apbsOptions['pdimeNX'], apbsOptions['pdimeNY'], apbsOptions['pdimeNZ']))
+        input.write('%sofrac %g\n' % (apbsOptions['tab'], apbsOptions['ofrac']))
+        if apbsOptions['asyncflag']:
+            input.write('%sasync %d\n' % (apbsOptions['tab'], apbsOptions['async']))
 
     if apbsOptions['calcType'] == "mg-manual":
         input.write('%sglen %g %g %g\n' % (apbsOptions['tab'], apbsOptions['glenX'], apbsOptions['glenY'], apbsOptions['glenZ']))
@@ -1509,9 +1525,16 @@ def pqrFileCreator(apbsOptions):
     input.write('%stemp %g\n' % (apbsOptions['tab'], apbsOptions['temperature']))
     input.write('%scalcenergy %s\n' % (apbsOptions['tab'], apbsOptions['calcEnergy']))
     input.write('%scalcforce %s\n' % (apbsOptions['tab'], apbsOptions['calcForce']))
-    if apbsOptions.has_key('mobileIonSpeciesCharge') and apbsOptions.has_key('conc') and apbsOptions.has_key('radius'):
-        input.write('%sion %d %g %g\n' % (apbsOptions['tab'], apbsOptions['mobileIonSpeciesCharge'], apbsOptions['mobileIonSpeciesConcentration'], apbsOptions['mobileIonSpeciesRadius']))
-
+    for i in range(0,3):
+        chStr = 'charge%i' % i
+        concStr = 'conc%i' % i
+        radStr = 'radius%i' % i
+        if apbsOptions.has_key(chStr) and apbsOptions.has_key(concStr) and apbsOptions.has_key(radStr):
+            #ion charge {charge} conc {conc} radius {radius}
+            input.write('%sion charge %d conc %g radius %g\n' % (apbsOptions['tab'], 
+                                                                 apbsOptions[chStr], 
+                                                                 apbsOptions[concStr], 
+                                                                 apbsOptions[radStr]))
 
     if apbsOptions['writeCharge']:
         input.write('%swrite charge %s %s-charge\n' % (apbsOptions['tab'], apbsOptions['writeFormat'], apbsOptions['writeStem']))
