@@ -46,8 +46,6 @@
 
 __date__  = "5 April 2010"
 __author__ = "Todd Dolinsky, Nathan Baker, Jens Nielsen, Paul Czodrowski, Jan Jensen, Samir Unni, Yong Huang"
-__version__ = "1.8"
-
 
 import string
 import sys
@@ -72,10 +70,36 @@ from src.server import *
 from src.hydrogens import *
 from src.aconf import *
 from StringIO import *
+from src.errors import PDB2PQRError
+
+__version__ = PDB2PQR_VERSION
 
 import extensions
 
-def printPQRHeader(atomlist, reslist, charge, ff, warnings, pH, ffout):
+def getOldHeader(pdblist):
+    oldHeader = StringIO()
+    headerTypes = (HEADER, TITLE, COMPND, SOURCE, 
+                   KEYWDS, EXPDTA, AUTHOR, REVDAT, 
+                   JRNL, REMARK, SPRSDE, NUMMDL)
+    for pdbObj in pdblist:
+        if not isinstance(pdbObj,headerTypes):
+            break
+        
+        oldHeader.write(str(pdbObj))
+        oldHeader.write('\n')
+        
+    return oldHeader.getvalue()
+
+def printPQRHeader(pdblist, 
+                   atomlist, 
+                   reslist, 
+                   charge, 
+                   ff, 
+                   warnings, 
+                   pH, 
+                   ffout, 
+                   cl_args, 
+                   include_old_header = False):
     """
         Print the header for the PQR file
 
@@ -87,7 +111,9 @@ def printPQRHeader(atomlist, reslist, charge, ff, warnings, pH, ffout):
             charge:   The total charge on the protein (float)
             ff:       The forcefield name (string)
             warnings: A list of warnings generated from routines (list)
-            options:  A dictionary of command lnie options (float)
+            pH :  pH value, if any. (float)
+            ffout :  ff used for naming scheme (string)
+            cl_args : the command line argument used when running pdb2pqr (string)
         Returns
             header:   The header for the PQR file (string)
     """
@@ -97,6 +123,10 @@ def printPQRHeader(atomlist, reslist, charge, ff, warnings, pH, ffout):
         ff = ff.upper()
     header = "REMARK   1 PQR file generated by PDB2PQR (Version %s)\n" % __version__
     header = header + "REMARK   1\n"
+    if cl_args is not None:
+        header +=     "REMARK   1 Command line used to generate this file:\n"
+        header +=     "REMARK   1 " + cl_args + "\n"
+        header = header + "REMARK   1\n"
     header = header + "REMARK   1 Forcefield Used: %s\n" % ff
     if not ffout is None:
         header = header + "REMARK   1 Naming Scheme Used: %s\n" % ffout
@@ -132,7 +162,13 @@ def printPQRHeader(atomlist, reslist, charge, ff, warnings, pH, ffout):
         header += "REMARK   5\n"
     header += "REMARK   6 Total charge on this protein: %.4f e\n" % charge
     header += "REMARK   6\n"
-
+    
+    if include_old_header:
+        header += "REMARK   7 Original PDB header follows\n"
+        header += "REMARK   7\n"
+    
+        header += getOldHeader(pdblist)
+    
     return header
 
 def runPDB2PQR(pdblist, ff,
@@ -153,7 +189,9 @@ def runPDB2PQR(pdblist, ff,
                typemap = False,
                userff = None,
                usernames = None,
-               ffout = None):
+               ffout = None,
+               commandLine=None,
+               include_old_header=False):
     """
         Run the PDB2PQR Suite
 
@@ -183,6 +221,8 @@ def runPDB2PQR(pdblist, ff,
             usernames:     The user created names file to use. Required if using userff.
             ffout:         Instead of using the standard canonical naming scheme for residue and atom names,  +
                            use the names from the given forcefield
+            commandLine:   command line used (if any) to launch the program. Included in output header.
+            include_old_header: Include most of the PDB header in output.
             
         Returns
             header:  The PQR file header (string)
@@ -197,12 +237,7 @@ def runPDB2PQR(pdblist, ff,
     Lig = None
     atomcount = 0   # Count the number of ATOM records in pdb
     
-    period = string.rfind(outname,".")
-    
-    if period > 0: 
-        outroot = outname[0:period]
-    else: 
-        outroot = outname
+    outroot = utilities.getPQRBaseFileName(outname)
 
     if not ph is None:
         pka = True
@@ -302,7 +337,6 @@ def runPDB2PQR(pdblist, ff,
             myhydRoutines.initializeFullOptimization()
             myhydRoutines.optimizeHydrogens()
         else:
-            #myhydRoutines = hydrogenRoutines(myRoutines)
             myhydRoutines.initializeWaterOptimization()
             myhydRoutines.optimizeHydrogens()
 
@@ -379,7 +413,9 @@ def runPDB2PQR(pdblist, ff,
             myNameScheme = myForcefield
         myRoutines.applyNameScheme(myNameScheme)
 
-    header = printPQRHeader(misslist, reslist, charge, ff, myRoutines.getWarnings(), ph, ffout)
+    header = printPQRHeader(pdblist, misslist, reslist, charge, ff, 
+                            myRoutines.getWarnings(), ph, ffout, commandLine, 
+                            include_old_header=include_old_header)
     lines = myProtein.printAtoms(hitlist, chain)
 
     # Determine if any of the atoms in misslist were ligands
@@ -393,8 +429,10 @@ def runPDB2PQR(pdblist, ff,
     # Process the extensions
     for ext in selectedExtensions:
         module = extensions.extDict[ext]
-        tempRoutines = copy.deepcopy(myRoutines)
-        module.run_extension(tempRoutines, outroot, extensionOptions)
+        #TODO: figure out a way to do this without crashing...
+        #tempRoutines = copy.deepcopy(myRoutines)
+        module.run_extension(myRoutines, outroot, extensionOptions)
+        
 
     if verbose:
         print "Total time taken: %.2f seconds\n" % (time.time() - start)
@@ -421,10 +459,10 @@ def mainCommand(argv):
     parser = OptionParser(description=description, usage=usage, version='%prog (Version ' + __version__ + ')')
     
 
-    group = OptionGroup(parser,"Mandatory options", "One of the following options must be used.")
+    group = OptionGroup(parser,"Manditory options", "One of the following options must be used.")
     group.add_option('--ff', dest='ff', metavar='FIELD_NAME', choices=validForcefields,
-                      help='The forcefield to use - currently AMBER, ' +
-                           'CHARMM, PARSE, TYL06, PEOEPB and SWANSON ' +
+                      help='The forcefield to use - currently amber, ' +
+                           'charmm, parse, tyl06, peoepb and swanson ' +
                            'are supported.')
     
     group.add_option('--userff', dest='userff', metavar='USER_FIELD_FILE', 
@@ -452,9 +490,8 @@ def mainCommand(argv):
     
     group.add_option('--ffout', dest='ffout', metavar='FIELD_NAME',choices=validForcefields,
                       help='Instead of using the standard canonical naming scheme for residue and atom names, ' +
-                           'use the names from the given forcefield - currently AMBER, ' +
-                           'CHARMM, PARSE, TYL06, PEOEPB and SWANSON ' +
-                           'are supported.')
+                           'use the names from the given forcefield - currently amber, ' +
+                           'charmm, parse, tyl06, peoepb and swanson are supported.')
     
     group.add_option('--usernames', dest='usernames', metavar='USER_NAME_FILE', 
                       help='The user created names file to use. Required if using --userff')
@@ -474,50 +511,56 @@ def mainCommand(argv):
                       help='Create Typemap output.')
     
     group.add_option('--neutraln', dest='neutraln', action='store_true', default=False,
-                      help='Make the N-terminus of this protein neutral (default is charged). ' +
+                      help='Make the N-terminus of this protein neutral (default is charged). '
                            'Requires PARSE force field.')  
     
     group.add_option('--neutralc', dest='neutralc', action='store_true', default=False,
-                      help='Make the C-terminus of this protein neutral (default is charged). ' +
-                           'Requires PARSE force field.')  
+                      help='Make the C-terminus of this protein neutral (default is charged). '
+                           'Requires PARSE force field.') 
 
     group.add_option('-v', '--verbose', dest='verbose', action='store_true', default=False,
                       help='Print information to stdout.')
+    
+    group.add_option('--include_header', dest='include_header', action='store_true', default=False,
+                      help='Include pdb header in pqr file. '
+                           'WARNING: The resulting PQR file will not with APBS versions prior to 1.5')
     parser.add_option_group(group)
     
     
-    propkaroup = OptionGroup(parser,"propka options")
+    propkaroup = OptionGroup(parser,"proPKA options")
     
     propkaroup.add_option('--with-ph', dest='pH', action='store', type='float',
                       help='Use propka to calculate pKas and apply them to the molecule given the pH value. ' +
                            'Actual PropKa results will be output to <output-path>.propka.')
     
-    propkaroup.add_option("--reference", dest="reference", default="neutral", 
-           help="setting which reference to use for stability calculations [neutral/low-pH]")
+    propkaroup.add_option("--reference", dest="reference", default="neutral", choices=('neutral','low-pH'), 
+           help="Setting which reference to use for stability calculations. See PROPKA 3.0 documentation.")
+    
+    propkaroup.add_option('--propka-verbose', dest='propka_verbose', action='store_true', default=False,
+                      help='Print extra proPKA information to stdout. '
+                           'WARNING: This produces an incredible level of output.')
     
     parser.add_option_group(propkaroup)
     
     
     extensions.setupExtensionsOptions(parser)
     
-    (options, args) = parser.parse_args() 
+    (options, args) = parser.parse_args(argv[1:]) 
+    
+    commandLine = ' '.join(argv[1:])
     
     if len(args) != 2:
-        parser.error('Incorrect number (%d) of arguments!\nargv: %s, args: %s' % (len(args),argv, args))   
+        parser.error('Incorrect number (%d) of arguments!\nargs: %s' % (len(args), args))   
 
-    # Append Numeric/Numpy path to sys.path if the user specified a non-standard location during configuration
-    sys.argv=argv
-    package_path = PACKAGE_PATH
-    if package_path != "":
-        sys.path.extend(package_path.split(":"))
-       
     propkaOpts = None 
     if (not options.pH is None): 
         if(options.pH < 0.0 or options.pH > 14.0):
             parser.error('%i is not a valid pH!  Please choose a pH between 0.0 and 14.0.' % options.pH)
         
         #build propka options
-        propkaOpts = utilities.createPropkaOptions(options.pH, options.verbose)
+        propkaOpts = utilities.createPropkaOptions(options.pH, 
+                                                   verbose=options.propka_verbose, 
+                                                   reference=options.reference)
         
     if options.assign_only or options.clean:
         options.debump = options.optflag = False
@@ -555,20 +598,23 @@ def mainCommand(argv):
         except IOError:
             parser.error('Unable to find ligand file %s!' % options.ligand)
 
-    if options.neutraln and (options.ff != 'parse' or not options.userff is None):
+    if options.neutraln and (options.ff is None or options.ff.lower() != 'parse'):
         parser.error('--neutraln option only works with PARSE forcefield!')
         
-    if options.neutralc and (options.ff != 'parse' or not options.userff is None):
+    if options.neutralc and (options.ff is None or options.ff.lower() != 'parse'):
         parser.error('--neutralc option only works with PARSE forcefield!')
 
-    text =  "\n--------------------------\n"
-    text += "PDB2PQR - a Python-based structural conversion utility\n"
-    text += "--------------------------\n"
-    text += "Please cite your use of PDB2PQR as:\n"
-    text += "  Dolinsky TJ, Nielsen JE, McCammon JA, Baker NA.\n"
-    text += "  PDB2PQR: an automated pipeline for the setup, execution,\n"
-    text += "  and analysis of Poisson-Boltzmann electrostatics calculations.\n"
-    text += "  Nucleic Acids Research 32 W665-W667 (2004).\n\n"
+    text =  """
+--------------------------
+PDB2PQR - a Python-based structural conversion utility
+--------------------------
+Please cite your use of PDB2PQR as:
+  Dolinsky TJ, Nielsen JE, McCammon JA, Baker NA.
+  PDB2PQR: an automated pipeline for the setup, execution,
+  and analysis of Poisson-Boltzmann electrostatics calculations.
+  Nucleic Acids Research 32 W665-W667 (2004).
+
+"""
     sys.stdout.write(text)
             
     path = args[0]
@@ -597,26 +643,32 @@ def mainCommand(argv):
     # get rid of the userff and username arguments to this function.
     # This would also do away with the redundent checks and such in 
     # the Forcefield constructor.
-    header, lines, missedligands = runPDB2PQR(pdblist, 
-                                              options.ff, 
-                                              outname = options.outname,
-                                              ph = options.pH,
-                                              verbose = options.verbose,
-                                              selectedExtensions = options.active_extensions,
-                                              propkaOptions = propkaOpts,
-                                              extensionOptions = extensionOpts,
-                                              clean = options.clean,
-                                              neutraln = options.neutraln,
-                                              neutralc = options.neutralc,
-                                              ligand = options.ligand,
-                                              assign_only = options.assign_only,
-                                              chain = options.chain,
-                                              debump = options.debump,
-                                              opt = options.opt,
-                                              typemap = options.typemap,
-                                              userff = userfffile,
-                                              usernames = usernamesfile,
-                                              ffout = options.ffout)
+    try:
+        header, lines, missedligands = runPDB2PQR(pdblist, 
+                                                  options.ff, 
+                                                  outname = options.outname,
+                                                  ph = options.pH,
+                                                  verbose = options.verbose,
+                                                  selectedExtensions = options.active_extensions,
+                                                  propkaOptions = propkaOpts,
+                                                  extensionOptions = extensionOpts,
+                                                  clean = options.clean,
+                                                  neutraln = options.neutraln,
+                                                  neutralc = options.neutralc,
+                                                  ligand = options.ligand,
+                                                  assign_only = options.assign_only,
+                                                  chain = options.chain,
+                                                  debump = options.debump,
+                                                  opt = options.opt,
+                                                  typemap = options.typemap,
+                                                  userff = userfffile,
+                                                  usernames = usernamesfile,
+                                                  ffout = options.ffout,
+                                                  commandLine = commandLine,
+                                                  include_old_header = options.include_header)
+    except PDB2PQRError as er:
+        print er
+        sys.exit(1)
     
     # Print the PQR file
     outfile = open(outpath,"w")
@@ -642,7 +694,7 @@ def mainCommand(argv):
         size.parseInput(outpath)
         size.runPsize(outpath)
         async = 0 # No async files here!
-        input = inputgen.Input(outpath, size, method, async)
+        input = inputgen.Input(outpath, size, method, async, potdx=True)
         input.printInputFiles()
         input.dumpPickle()
 
