@@ -54,6 +54,7 @@ import string
 from pdb import *
 from utilities import *
 from quatfit import *
+from errors import PDBInternalError
 
 class Chain:
     """
@@ -94,7 +95,7 @@ class Chain:
                 return item
             except AttributeError:
                 message = 'Unable to get object "%s" in class Chain' % name
-                raise ValueError, message
+                raise PDBInternalError(message)
 
     def addResidue(self, residue):
         """
@@ -215,9 +216,9 @@ class Residue:
 
     def __str__(self):
         """
-            Basic string representation for debugging
+            Basic string representation
         """
-        text = "%s %s %i" % (self.name, self.chainID, self.resSeq)
+        text = "%s %s %i%s" % (self.name, self.chainID, self.resSeq, self.iCode)
         return text
     
     #TODO: Kill this in a fire.
@@ -247,7 +248,7 @@ class Residue:
             return item
         except AttributeError:
             message = "Unable to access object \"%s\" in class Residue" % name
-            raise ValueError, message
+            raise PDBInternalError(message)
 
     def set(self, name, value):
         """
@@ -281,7 +282,7 @@ class Residue:
                 setattr(self, name, value)
             except AttributeError:
                 message = "Unable to set object \"%s\" in class Residue" % name
-                raise ValueError, message
+                raise PDBInternalError(message)
 
     def update_terminus_status(self):
         """Update the isNterms and isCterm flags"""
@@ -576,7 +577,7 @@ class Atom(ATOM):
         if type == "ATOM" or type == "HETATM":
             self.type = type
         else:
-            raise ValueError, "Invalid atom type %s (Atom Class IN structures.py)!"
+            raise PDBInternalError("Invalid atom type %s (Atom Class IN structures.py)!" % type)
         self.serial = atom.serial
         self.name = atom.name
         self.altLoc = atom.altLoc
@@ -771,7 +772,7 @@ class Atom(ATOM):
             return item
         except AttributeError:
             message = "Unable to access object \"%s\" in class Atom" % name
-            raise ValueError, message
+            raise PDBInternalError(message)
 
     def set(self, name, value):
         """
@@ -807,7 +808,7 @@ class Atom(ATOM):
             setattr(self, name, value)
         except AttributeError:
             message = "Unable to set object \"%s\" in class Atom" % name
-            raise ValueError, message   
+            raise PDBInternalError(message)   
 
     def getCoords(self):
         """
